<?PHP
	class Languages {
		var $langList, $langCurrency;

		public function __construct() {
			$this->langList = [];
			$this->langCurrency = 'ru'; // Язык по умолчанию
		}

		/* Получаем текущую языковую таблицу */
		public function getList($lang = null) {
			if(is_null($lang)) {
				$lang = $this->langCurrency;
			}

			return $this->langList[$lang];
		}

		/* Возвращаем все ключи */
		public function getKeys($lang = 'ru') {
			return $this->langList[$lang];
		}
		
		/* Получить язык пользователя */
		public function Get() {
			if(isset($_SESSION['language'])) {
				return $_SESSION['language'];
			}

			return conf()->language;
		}

		/* Загрузка языкового файла */
		public function loadFromFile($lang, $filename) {
			$path = __DIR__ . '/../../languages/' . $filename;

			if(!file_exists($path)) {
				throw new Exception('Error loading the language file: ' . $path);
			}

			$texts = json_decode(file_get_contents($path), true);

			if($texts != null) {
				$this->langList[$lang] = $texts;
			}
			else {
				throw new Exception('Error loading the language file: ' . $path);
			}
		}

		public function Change($user_id) {
			try {
				$sth = pdo()->prepare('SELECT language FROM users WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $user_id]);

				if(!$sth->rowCount()) {
					return false;
				}

				$lang = $sth->fetch(PDO::FETCH_OBJ)->language;
				$this->langCurrency = $lang;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Установка языка */
		public function Set($lang) {
			if(array_key_exists($lang, $this->langList)) {
				$this->langCurrency = $lang;
				$_SESSION['language'] = $lang;

				if(isset($_SESSION['id'])) {
					try {
						$sth = pdo()->prepare('UPDATE users SET language=:language WHERE id=:id LIMIT 1');
						$sth->execute([':language' => $lang, ':id' => $_SESSION['id']]);
					}
					catch(Exception $e) {
						throw $e;
					}
				}
			}
			else {
				throw new Exception('Language ' . $lang . ' is not supported');
			}
		}

		/* Получаем текст */
		public function getText($key, $placeholders = []) {
			$keys = explode('.', $key);
			$text = $this->langList[$this->langCurrency];
			
			foreach($keys as $k) {
				if(isset($text[$k])) {
					$text = $text[$k];
				}
				else {
					throw new Exception('Translation not available for key: ' . $key);
				}
			}
			
			if(!empty($placeholders)) {
				if(!is_array($placeholders)) {
					$placeholders = [$placeholders];
				}

				$text = vsprintf($text, $placeholders);
			}
		
			return $text;
		}

		/* Получаем список языков */
		public function getOption($lang = 'ru') {
			try {
				$path = __DIR__ . '/../../languages/';
				$files = array_diff(scandir($path), ['.', '..']);
				$getLanguagesForJson = $this->getLanguagesForJson();

				tpl()->ClearCell('getOption');
				foreach($files as $key => $value) {
					$key = pathinfo($value, PATHINFO_FILENAME);

					if(empty($getLanguagesForJson[$key])) {
						continue;
					}

					tpl()->AddCell('getOption', tpl()->Set([
						'{text}' => $key,
						'{selected}' => ($lang == $key) ? 'selected' : '',
						'{name}' => $getLanguagesForJson[$key]['name']
					], '<option value="{text}" {selected}>{name}</option>'));
				}

				return tpl()->GetCell('getOption');
			}
			catch(Exception $e) {
				return tpl()->Set(['{text}' => $e->getMessage()], '<option disabled selected>{text}</option>');
			}
		}

		/* Загружаем языки */
		public function loadFromDir($dirs) {
			try {
				$files = array_diff(scandir($dirs), ['.', '..']);

				foreach($files as $key => $value) {
					$name = pathinfo($value, PATHINFO_FILENAME);
					$this->loadFromFile($name, $value);
				}
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Получаем активные языковые системы */
		public function getLanguagesForJson() {
			$path = getPath('configuration/system/languages.json');

			if(file_exists($path)) {
				return json_decode(file_get_contents($path), true);
			}
			else {
				return [
					'ru' => ['name' => 'Русский', 'enable' => true],
					'en' => ['name' => 'English', 'enable' => true]
				];
			}
		}

		/* Получаем данные языка */
		public function getData($lang = 'ru') {
			$getLanguagesForJson = $this->getLanguagesForJson();

			return [
				'name' => $getLanguagesForJson[$lang]['name'],
				'enable' => $getLanguagesForJson[$lang]['enable']
			];
		}

		// Применяем новые языки
		public function updateLanguageFiles() {
			try {
				$languagePath = __DIR__ . '/../../languages/';
				$updatePath = __DIR__ . '/../../updates/languages/';
		
				if(!is_dir($updatePath)) {
					return;
				}
		
				$updateFiles = array_diff(scandir($updatePath), ['.', '..']);
		
				foreach($updateFiles as $file) {
					$updateFilePath = $updatePath . $file;
					$languageFilePath = $languagePath . $file;
		
					if(!is_file($updateFilePath) || pathinfo($file, PATHINFO_EXTENSION) !== 'json') {
						continue;
					}
		
					$updates = json_decode(file_get_contents($updateFilePath), true);
					if(!is_array($updates)) {
						error_log("Ошибка загрузки JSON: $file");
						continue;
					}
		
					$currentLanguage = file_exists($languageFilePath) ? json_decode(file_get_contents($languageFilePath), true) : [];
					$currentLanguage = $this->arrayMergeRecursiveDistinct($currentLanguage, $updates);
					
					file_put_contents($languageFilePath, json_encode($currentLanguage, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
				}
		
				return true;
			}
			catch(Exception $e) {
				error_log("Ошибка в updateLanguageFiles: " . $e->getMessage());
			}
		}

		private function arrayMergeRecursiveDistinct(array $array1, array $array2) {
			foreach($array2 as $key => $value) {
				if(is_array($value) && isset($array1[$key]) && is_array($array1[$key])) {
					$array1[$key] = $this->arrayMergeRecursiveDistinct($array1[$key], $value);
				}
				else {
					$array1[$key] = $value;
				}
			}
		
			return $array1;
		}
	}