<?PHP
	class NowPayments {
		private $publicKey;
		private $secretKey;
		
		protected $urls = [
			'invoice' => 'https://api.nowpayments.io/v1/invoice',
			'status' => 'https://api.nowpayments.io/v1/status',
			'payment' => 'https://api.nowpayments.io/v1/payment',
			'currencies' => 'https://api.nowpayments.io/v1/currencies'
		];

		/**
		 * Конструктор класса
		 * 
		 * @param string|null $publicKey Публичный ключ API
		 * @param string|null $secretKey Секретный ключ API
		 */
		public function __construct($publicKey = null, $secretKey = null) {
			if (!is_null($publicKey)) {
				$this->setPublicKey($publicKey);
			}

			if (!is_null($secretKey)) {
				$this->setSecretKey($secretKey);
			}
		}

		/**
		 * Установка публичного ключа
		 * 
		 * @param string $key Публичный ключ API
		 */
		public function setPublicKey($key) {
			$this->publicKey = $key;
		}

		/**
		 * Установка секретного ключа
		 * 
		 * @param string $key Секретный ключ API
		 */
		public function setSecretKey($key) {
			$this->secretKey = $key;
		}

		/**
		 * Отправка запроса к API NOWPayments
		 * 
		 * @param string $url URL для запроса
		 * @param array $data Данные для отправки
		 * @return array Ответ от сервера в формате массива
		 */
		protected function sendRequest($url, $data = []) {
			$ch = curl_init($url);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, [
				'x-api-key: ' . $this->secretKey
			]);

			$response = curl_exec($ch);
			curl_close($ch);

			$responseData = json_decode($response, true);
			if (json_last_error() != JSON_ERROR_NONE) {
				throw new Exception('Ошибка декодирования JSON: ' . json_last_error_msg());
			}
			return $responseData;
		}

		/**
		 * Создание счета на оплату
		 * 
		 * @param float $amount Сумма для оплаты
		 * @param string $currency Валюта (например, 'USD')
		 * @param string $orderId ID заказа
		 * @param string $description Описание заказа
		 * @param string|null $payCurrency Валюта для оплаты, если отличается от $currency
		 * @return array Данные созданного счета
		 */
		public function createInvoice($amount, $currency, $orderId, $description, $payCurrency = null) {
			$data = [
				'price_amount' => $amount,
				'price_currency' => $currency,
				'order_id' => $orderId,
				'order_description' => $description,
				'ipn_callback_url' => 'https://' . $_SERVER['SERVER_NAME'] . '/pay/index?type=confirm&merchant=nowpayments',
				'success_url' => 'https://' . $_SERVER['SERVER_NAME'] . '/pay/index?type=fail',
				'cancel_url' => 'https://' . $_SERVER['SERVER_NAME'] . '/pay/index?type=success'
			];
		
			if ($payCurrency) {
				$data['pay_currency'] = $payCurrency;
			}
		
			return $this->sendRequest($this->urls['invoice'], $data);
		}

		/**
		 * Обработка уведомления об оплате (IPN)
		 * 
		 * @return array|bool Возвращает данные платежа если успешно и валидно, иначе false
		 */
		public function handleIPN() {
			if ($_SERVER['REQUEST_METHOD'] == 'POST') {
				$content = file_get_contents('php://input');
				$data = json_decode($content, true);

				if ($data && isset($data['payment_status'], $data['payment_id'], $data['hash'])) {
					// Валидация подписи
					$hash = hash_hmac('sha512', $data['payment_id'] . $data['payment_status'] . $data['price_amount'] . $data['price_currency'], $this->secretKey);
					if ($hash == $data['hash']) {
						return $data;
					}
				}
			}
			return false;
		}

		/**
		 * Проверка статуса платежа
		 * 
		 * @param string $paymentId ID платежа для проверки
		 * @return array Статус платежа
		 */
		public function checkPaymentStatus($paymentId) {
			$data = ['payment_id' => $paymentId];
			return $this->sendRequest($this->urls['payment'], $data);
		}

		/**
		 * Получение списка поддерживаемых валют
		 * 
		 * @return array Список валют
		 */
		public function getCurrencies() {
			return $this->sendRequest($this->urls['currencies']);
		}

		/**
		 * Проверка статуса API
		 * 
		 * @return array Статус API
		 */
		public function checkStatus() {
			return $this->sendRequest($this->urls['status']);
		}
	}